﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.ServiceModel.Activities;
using System.ServiceModel.Channels;
using System.ServiceModel;
using TakeThatWF.Core;
using System.Activities;

namespace TakeThatWF.Infrastructure
{
    public class CustomEndPoint : WorkflowHostingEndpoint
    {
        public CustomEndPoint(Binding binding, EndpointAddress epAddress)
            : base(typeof(IProxyService), binding, epAddress)
        {

        }

        /// <summary>
        /// Parse the passed in arguments and get the WF instacne GUID
        /// </summary>
        /// <param name="inputs">This is an array containing the input parameters on the method that was invoked. inputs[0] is first parameter and so on</param>
        /// <param name="operationContext"></param>
        /// <returns></returns>
        protected override Guid OnGetInstanceId(object[] inputs, OperationContext operationContext)
        {
            Guid id = Guid.Empty;
            string method = operationContext.IncomingMessageHeaders.Action;//This will be the name of the method that was invoked
            if (method.EndsWith(ProxyWFServiceActions.Action_InitiateWFInstance))
            {
                //do nothing
            }
            else if (method.EndsWith(ProxyWFServiceActions.Action_ResumeBookmarkedWF))
            {
                id = (Guid)inputs[0];//first param in the method signature of the interface member is GUID
            }
            else
                throw new InvalidOperationException(String.Format("You seem drunk or don't know where you are going. I don't have anything that corresponds to {0}", operationContext.IncomingMessageHeaders.Action));

            return id;
        }

        /// <summary>
        /// This is where we override the default logic and create our own WorkflowCreationContext. We parse the incoming request
        /// get the input parameters and pass them onto the workflow. This context can also handle workflow notifications
        /// </summary>
        /// <param name="inputs"></param>
        /// <param name="operationContext"></param>
        /// <param name="instanceId"></param>
        /// <param name="responseContext"></param>
        /// <returns></returns>
        protected override WorkflowCreationContext OnGetCreationContext(object[] inputs, OperationContext operationContext, Guid instanceId, WorkflowHostingResponseContext responseContext)
        {
            WorkflowCreationContext context = new WorkflowCreationContext();

            if (operationContext.IncomingMessageHeaders.Action.EndsWith(ProxyWFServiceActions.Action_InitiateWFInstance))
            {
                Dictionary<string, object> arguments = (Dictionary<string, object>)inputs[0];

                // Add arguments to the WorkflowArguments collection if any have been specified
                if (arguments != null && arguments.Count > 0)
                {
                    foreach (KeyValuePair<string, object> pair in arguments)
                        context.WorkflowArguments.Add(pair.Key, pair.Value);
                }

                // Reply to client with the InstanceId
                responseContext.SendResponse(instanceId, null);
            }
            else
                throw new InvalidOperationException(string.Format("Seriously quit drinking. I have no clue what \"{0}\" is", operationContext.IncomingMessageHeaders.Action));

            return context;
        }

        /// <summary>
        /// This is where we find a bookmark that was previously set and resume the execution of the workflow. The client passes in the
        /// Bookmark in the inputs collection.
        /// </summary>
        /// <param name="inputs"></param>
        /// <param name="operationContext"></param>
        /// <param name="responseContext"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        protected override Bookmark OnResolveBookmark(object[] inputs, OperationContext operationContext, WorkflowHostingResponseContext responseContext, out object value)
        {
            Bookmark bookmark = null;
            value = null;
            if (operationContext.IncomingMessageHeaders.Action.EndsWith(ProxyWFServiceActions.Action_ResumeBookmarkedWF))
            {
                bookmark = new Bookmark(inputs[1].ToString());
                //did the caller specify any instance state?
                if (inputs.Length > 2)
                    value = inputs[2];
            }
            else
                throw new InvalidOperationException(String.Format("There is no justification this time. I dont have {0}", operationContext.IncomingMessageHeaders.Action));
            return bookmark;
        }

    }
}